@@ -22,6 +22,7 @@ from requests.packages.urllib3.util.retry import Retry
 from dateutil.parser import parse as parse_date
 from dateutil.parser import ParserError
 
+import pylangacq
 from pylangacq.measures import _CLITIC, _get_ipsyn, _get_mlum, _get_mluw, _get_ttr
 from pylangacq.objects import Gra, Token, Utterance
 from pylangacq._clean_utterance import _clean_utterance
@@ -92,7 +93,7 @@ def _params_in_docstring(*params, class_method=True):
             For example, to work with the American English dataset Brown (containing
             data for the children Adam, Eve, and Sarah),
             you can pass in ``"Eve"`` here to only handle the data for Eve, since
-            the unzipped Brown data from CHILDES is in a directory structure of
+            the unzipped Brown data from CHILDES has a directory structure of
             ``Brown/Eve/xxx.cha`` for Eve's data.
             If this parameter is not specified or ``None`` is passed in (the default),
             such file path filtering does not apply.
@@ -127,6 +128,16 @@ def _params_in_docstring(*params, class_method=True):
             Pass in your own :class:`~pylangacq.Reader` subclass
             for new or modified behavior of the returned reader object."""
 
+    if "parallel" in params:
+        docstring += """
+        parallel : bool, optional
+            If ``True`` (the default), CHAT reading and parsing is parallelized
+            for speed-up, because in most cases multiple CHAT data files and/or strings
+            are being handled.
+            Under certain circumstances (e.g., your application is already parallelized
+            and further parallelization from within PyLangAcq might be undesirable),
+            you may like to consider setting this parameter to ``False``."""
+
     if not class_method:
         docstring = docstring.replace("\n        ", "\n    ")
 
@@ -193,10 +204,17 @@ class Reader:
         """Initialize an empty reader."""
         self._files = collections.deque()
 
-    def _parse_chat_strs(self, strs: List[str], file_paths: List[str]) -> None:
-        with cf.ProcessPoolExecutor() as executor:
+    def _parse_chat_strs(
+        self, strs: List[str], file_paths: List[str], parallel: bool
+    ) -> None:
+        if parallel:
+            with cf.ProcessPoolExecutor() as executor:
+                self._files = collections.deque(
+                    executor.map(self._parse_chat_str, strs, file_paths)
+                )
+        else:
             self._files = collections.deque(
-                executor.map(self._parse_chat_str, strs, file_paths)
+                self._parse_chat_str(s, f) for s, f in zip(strs, file_paths)
             )
 
     def __len__(self):
@@ -240,13 +258,13 @@ class Reader:
         """Remove all data from this reader."""
         self._files = collections.deque()
 
-    def _append(self, left_or_right, reader: "Reader") -> None:
+    def _append(self, left_or_right, reader: "pylangacq.Reader") -> None:
         func = "extendleft" if left_or_right == "left" else "extend"
-        if type(reader) != Reader:
+        if not issubclass(reader.__class__, Reader):
             raise TypeError(f"not a Reader object: {type(reader)}")
         getattr(self._files, func)(reader._files)
 
-    def append(self, reader: "Reader") -> None:
+    def append(self, reader: "pylangacq.Reader") -> None:
         """Append data from another reader.
 
         New data is appended as-is with no filtering of any sort,
@@ -259,7 +277,7 @@ class Reader:
         """
         self._append("right", reader)
 
-    def append_left(self, reader: "Reader") -> None:
+    def append_left(self, reader: "pylangacq.Reader") -> None:
         """Left-append data from another reader.
 
         New data is appended as-is with no filtering of any sort,
@@ -272,18 +290,18 @@ class Reader:
         """
         self._append("left", reader)
 
-    def _extend(self, left_or_right, readers: "Iterable[Reader]") -> None:
+    def _extend(self, left_or_right, readers: "Iterable[pylangacq.Reader]") -> None:
         # Loop through each object in ``readers`` explicitly, so that we have
         # a chance to check that the object is indeed a Reader instance.
         new_files = []
         for reader in readers:
-            if type(reader) != Reader:
+            if not issubclass(reader.__class__, Reader):
                 raise TypeError(f"not a Reader object: {type(reader)}")
             new_files.extend(reader._files)
         func = "extendleft" if left_or_right == "left" else "extend"
         getattr(self._files, func)(new_files)
 
-    def extend(self, readers: "Iterable[Reader]") -> None:
+    def extend(self, readers: "Iterable[pylangacq.Reader]") -> None:
         """Extend data from other readers.
 
         New data is appended as-is with no filtering of any sort,
@@ -298,7 +316,7 @@ class Reader:
         # a chance to check that the object is indeed a Reader instance.
         self._extend("right", readers)
 
-    def extend_left(self, readers: "Iterable[Reader]") -> None:
+    def extend_left(self, readers: "Iterable[pylangacq.Reader]") -> None:
         """Left-extend data from other readers.
 
         New data is appended as-is with no filtering of any sort,
@@ -311,12 +329,12 @@ class Reader:
         """
         self._extend("left", readers)
 
-    def _pop(self, left_or_right) -> "Reader":
+    def _pop(self, left_or_right) -> "pylangacq.Reader":
         func = "popleft" if left_or_right == "left" else "pop"
         file_ = getattr(self._files, func)()
         return self._get_reader_from_files([file_])
 
-    def pop(self) -> "Reader":
+    def pop(self) -> "pylangacq.Reader":
         """Drop the last data file from the reader and return it as a reader.
 
         Returns
@@ -325,7 +343,7 @@ class Reader:
         """
         return self._pop("right")
 
-    def pop_left(self) -> "Reader":
+    def pop_left(self) -> "pylangacq.Reader":
         """Drop the first data file from the reader and return it as a reader.
 
         Returns
@@ -334,6 +352,32 @@ class Reader:
         """
         return self._pop("left")
 
+    @_params_in_docstring("match", "exclude")
+    def filter(self, match: str = None, exclude: str = None) -> "pylangacq.Reader":
+        """Return a new reader filtered by file paths.
+
+        Parameters
+        ----------
+
+
+        Returns
+        -------
+        :class:`pylangacq.Reader`
+
+        Raises
+        ------
+        TypeError
+            If neither ``match`` nor ``exclude`` is specified.
+        """
+        if not match and not exclude:
+            raise TypeError("At least one of {match, exclude} must be specified")
+        reader = self.__class__()
+        file_paths = set(self._filter_file_paths(self.file_paths(), match, exclude))
+        reader._files = collections.deque(
+            f for f in self._files if f.file_path in file_paths
+        )
+        return reader
+
     @staticmethod
     def _flatten(item_type, nested) -> Union[List, Set]:
         if item_type == list:
@@ -846,7 +890,10 @@ class Reader:
             return self._flatten(collections.Counter, result_by_files)
 
     @classmethod
-    def from_strs(cls, strs: List[str], ids: List[str] = None) -> "Reader":
+    @_params_in_docstring("parallel")
+    def from_strs(
+        cls, strs: List[str], ids: List[str] = None, parallel: bool = True
+    ) -> "pylangacq.Reader":
         """Instantiate a reader from in-memory CHAT data strings.
 
         Parameters
@@ -873,18 +920,19 @@ class Reader:
                 f"strs and ids must have the same size: {len(strs)} and {len(ids)}"
             )
         reader = cls()
-        reader._parse_chat_strs(strs, ids)
+        reader._parse_chat_strs(strs, ids, parallel)
         return reader
 
     @classmethod
-    @_params_in_docstring("match", "exclude", "encoding")
+    @_params_in_docstring("match", "exclude", "encoding", "parallel")
     def from_files(
         cls,
         paths: List[str],
         match: str = None,
         exclude: str = None,
         encoding: str = _ENCODING,
-    ) -> "Reader":
+        parallel: bool = True,
+    ) -> "pylangacq.Reader":
         """Instantiate a reader from local CHAT data files.
 
         Parameters
@@ -903,23 +951,31 @@ class Reader:
             with open(path, encoding=encoding) as f:
                 return f.read()
 
-        paths = list(paths)
+        paths = cls._filter_file_paths(paths, match, exclude)
 
+        if parallel:
+            with cf.ThreadPoolExecutor() as executor:
+                strs = list(executor.map(_open_file, paths))
+        else:
+            strs = [_open_file(p) for p in paths]
+
+        return cls.from_strs(strs, paths, parallel=parallel)
+
+    @staticmethod
+    def _filter_file_paths(
+        paths: List[str], match: str = None, exclude: str = None
+    ) -> List[str]:
+        paths = list(paths)
         if match:
             regex = re.compile(match)
             paths = [p for p in paths if regex.search(p)]
-
         if exclude:
             regex = re.compile(exclude)
             paths = [p for p in paths if not regex.search(p)]
-
-        with cf.ThreadPoolExecutor() as executor:
-            strs = list(executor.map(_open_file, paths))
-
-        return cls.from_strs(strs, paths)
+        return paths
 
     @classmethod
-    @_params_in_docstring("match", "exclude", "extension", "encoding")
+    @_params_in_docstring("match", "exclude", "extension", "encoding", "parallel")
     def from_dir(
         cls,
         path: str,
@@ -927,7 +983,8 @@ class Reader:
         exclude: str = None,
         extension: str = _CHAT_EXTENSION,
         encoding: str = _ENCODING,
-    ) -> "Reader":
+        parallel: bool = True,
+    ) -> "pylangacq.Reader":
         """Instantiate a reader from a local directory with CHAT data files.
 
         Parameters
@@ -951,11 +1008,17 @@ class Reader:
                     continue
                 file_paths.append(os.path.join(dirpath, filename))
         return cls.from_files(
-            sorted(file_paths), match=match, exclude=exclude, encoding=encoding
+            sorted(file_paths),
+            match=match,
+            exclude=exclude,
+            encoding=encoding,
+            parallel=parallel,
         )
 
     @classmethod
-    @_params_in_docstring("match", "exclude", "extension", "allow_remote", "encoding")
+    @_params_in_docstring(
+        "match", "exclude", "extension", "allow_remote", "encoding", "parallel"
+    )
     def from_zip(
         cls,
         path: str,
@@ -964,7 +1027,8 @@ class Reader:
         extension: str = _CHAT_EXTENSION,
         allow_remote: bool = True,
         encoding: str = _ENCODING,
-    ) -> "Reader":
+        parallel: bool = True,
+    ) -> "pylangacq.Reader":
         """Instantiate a reader from a local or remote ZIP file.
 
         Parameters
@@ -1002,6 +1066,7 @@ class Reader:
                 exclude=exclude,
                 extension=extension,
                 encoding=encoding,
+                parallel=parallel,
             )
 
         # Unzipped files from `.from_zip` have the unwieldy temp dir in the file path.
@@ -1308,7 +1373,7 @@ def read_chat(
     exclude: str = None,
     encoding: str = _ENCODING,
     cls: type = Reader,
-) -> Reader:
+) -> "pylangacq.Reader":
     """Create a reader of CHAT data.
 
     Parameters
